==> replacer-2.0.0%2B1.21.%286-8%29-Fabric-sources.jar: eva/replacer/mixin/client/MultiPlayerGameModeMixin.java <==
package eva.replacer.mixin.client;

import com.llamalad7.mixinextras.sugar.Local;
import eva.replacer.RePlacerClient;
import eva.replacer.mixin.client.access.BlockHitResultAccess;
import eva.replacer.mixin.client.access.MultiPlayerGameModeInvoker;
import eva.replacer.util.BuildHolder;
import eva.replacer.util.BuildInProgress;
import eva.replacer.util.RelPos;
import net.fabricmc.api.EnvType;
import net.fabricmc.api.Environment;
import net.minecraft.class_1268;
import net.minecraft.class_1269;
import net.minecraft.class_1747;
import net.minecraft.class_1750;
import net.minecraft.class_2338;
import net.minecraft.class_2561;
import net.minecraft.class_2885;
import net.minecraft.class_310;
import net.minecraft.class_3468;
import net.minecraft.class_3965;
import net.minecraft.class_636;
import net.minecraft.class_638;
import net.minecraft.class_7204;
import net.minecraft.class_746;
import org.apache.commons.lang3.mutable.MutableObject;
import org.jetbrains.annotations.NotNull;
import org.spongepowered.asm.mixin.Final;
import org.spongepowered.asm.mixin.Mixin;
import org.spongepowered.asm.mixin.Shadow;
import org.spongepowered.asm.mixin.Unique;
import org.spongepowered.asm.mixin.injection.At;
import org.spongepowered.asm.mixin.injection.Inject;
import org.spongepowered.asm.mixin.injection.Redirect;
import org.spongepowered.asm.mixin.injection.callback.CallbackInfo;
import org.spongepowered.asm.mixin.injection.callback.CallbackInfoReturnable;

import static eva.replacer.RePlacerClient.*;
import static eva.replacer.config.RePlacerConfig.*;
import static eva.replacer.util.BuildHolder.setBaseDir;
import static eva.replacer.util.BuildHolder.setFacing;
import static eva.replacer.util.BuildInProgress.*;
import static eva.replacer.util.RelPos.origin;

@Environment(EnvType.CLIENT)
@Mixin(class_636.class)
public abstract class MultiPlayerGameModeMixin {
    @Final @Shadow @NotNull private class_310 minecraft;
    @Unique @NotNull private final class_636 gameMode = (class_636) (Object) this;
    @Unique private class_746 player;
    @Unique private int waiter;

    @Inject(
            method = "tick",
            at = @At("HEAD")
    )
    private void placeAgain(CallbackInfo ci) {
        assert minecraft.field_1724 != null;
        if (player == null) player = minecraft.field_1724;
        if (rePlacing()) {
            if ((!isServer() || isReadyForBlock()) && player.method_5998(context().method_20287()).method_7909() instanceof class_1747 item) {
                if (!BuildInProgress.isEmpty()) {
                    waiter = 0;
                    RelPos pos = next();
                    if (player.method_56093(pos.pos(), 1)) {
                        assert context().method_8036() != null;
                        class_3965 result = BuildInProgress.place(pos, context().method_8036(), item);
                        if (!((BlockHitResultAccess) result).isMiss()) {
                            setReadyForBlock(false);
                            player.method_7259(class_3468.field_15372.method_14956(player.method_5998(context().method_20287()).method_7909()));
                        }
                    } /*else {
                        BuildInProgress.clear();
                        rePlacing(false);
                    }*/
                    return;
                } else if (!isClear()) BuildInProgress.clear();
                rePlacing(false);
            } else {
                if (waiter >= 5) {
                    if (!isClear()) BuildInProgress.clear();
                    rePlacing(false);
                    return;
                }
                waiter++;
            }
        }
    }

    @Redirect(
            method = "useItemOn",
            at = @At(
                    value = "INVOKE",
                    target = "Lnet/minecraft/client/multiplayer/MultiPlayerGameMode;startPrediction(Lnet/minecraft/client/multiplayer/ClientLevel;Lnet/minecraft/client/multiplayer/prediction/PredictiveAction;)V"
            )
    )
    private void reUseItemOn(
            class_636 instance,
            class_638 level,
            class_7204 action,
            @Local MutableObject<class_1269> mutableObject,
            @Local(argsOnly = true) class_1268 hand,
            @Local(argsOnly = true) class_3965 result
    ) {
        if (rePlacing()) return;

        if (!(player.method_5998(hand).method_7909() instanceof class_1747) || reCording || rePlacing() || !isHeldOrToggled()) {
            ((MultiPlayerGameModeInvoker) gameMode).invokeStartPrediction(this.minecraft.field_1687, i -> {
                mutableObject.setValue(((MultiPlayerGameModeInvoker) gameMode).invokePerformUseItemOn(player, hand, result));
                return new class_2885(hand, result, i);
            });
            if (mutableObject.getValue() instanceof class_1269.class_9860 && reCording) {
                class_1750 context = new class_1750(
                        player,
                        hand,
                        player.method_5998(hand),
                        result
                );
                if (buildSaver(context)) {
                    assert context.method_8036() != null;
                    context.method_8036().method_7353(class_2561.method_43470("Central block saved!"), true);
                }
            }
        } else {
            class_1750 context = new class_1750(
                    player,
                    hand,
                    player.method_5998(hand),
                    result
            );
            BuildHolder holder = isQuickAccess() ? getQuickBuild() : getBuild();
            if (isQuickAccess()) setQuickAccess(-1);
            if (holder == null) {
                RePlacerClient.LOGGER.info("Failed to get build!");
                player.method_7353(class_2561.method_43470("Failed to get build!"), true);
                player.method_7353(class_2561.method_43470("Writing default build."), false);
                return;
            }
            rePlacing(true);
            RelPos.setBase(context.method_17698(), context.method_8037());
            try {
                if (origin.equals(new RelPos(holder.x()[0], holder.y()[0], holder.z()[0])))
                    ((MultiPlayerGameModeInvoker) gameMode).invokeStartPrediction(this.minecraft.field_1687, i -> {
                        mutableObject.setValue(((MultiPlayerGameModeInvoker) gameMode).invokePerformUseItemOn(player, hand, result));
                        return new class_2885(hand, result, i);
                    });
                else {
                    player.method_6104(hand);
                    mutableObject.setValue(class_1269.field_5812);
                }
                if (!(mutableObject.getValue() == null) && !(mutableObject.getValue() instanceof class_1269.class_9860)) {
                    rePlacing(false);
                    return;
                }
                if (isSnap()) setBaseDir(context.method_8038());
                if (isSpin()) setFacing(player.method_5720(), context.method_8038());
                //Formerly compat mode
                BuildInProgress.pass(context);
                setReadyForBlock(false);
                holder.rotateEach(pos -> {
                    if (player.method_56093(pos.pos(), 1)
                            && level.method_8320(pos.pos()).method_26166(context)
                    )
                        pass(pos);
                });

            } catch (Exception ignored) {}
        }
    }

    @Inject(
            method = "destroyBlock",
            at = @At(
                    value = "INVOKE",
                    target = "Lnet/minecraft/world/level/block/Block;destroy(Lnet/minecraft/world/level/LevelAccessor;Lnet/minecraft/core/BlockPos;Lnet/minecraft/world/level/block/state/BlockState;)V"
            )
    )
    private void removeBlock(class_2338 pos, CallbackInfoReturnable<Boolean> cir) {
        if (reCording) buildRemover(pos);
    }
}

==> replacer-2.1.1%2B1.21.%286-8%29-Fabric-sources.jar: eva/replacer/mixin/client/MultiPlayerGameModeMixin.java <==
package eva.replacer.mixin.client;

import com.llamalad7.mixinextras.sugar.Local;
import eva.replacer.RePlacerClient;
import eva.replacer.mixin.client.access.BlockHitResultAccess;
import eva.replacer.mixin.client.access.MultiPlayerGameModeInvoker;
import eva.replacer.util.BuildHolder;
import eva.replacer.util.BuildInProgress;
import eva.replacer.util.RelPos;
import net.fabricmc.api.EnvType;
import net.fabricmc.api.Environment;
import net.minecraft.class_1268;
import net.minecraft.class_1269;
import net.minecraft.class_1747;
import net.minecraft.class_1750;
import net.minecraft.class_2338;
import net.minecraft.class_2561;
import net.minecraft.class_2885;
import net.minecraft.class_310;
import net.minecraft.class_3468;
import net.minecraft.class_3965;
import net.minecraft.class_636;
import net.minecraft.class_638;
import net.minecraft.class_7204;
import net.minecraft.class_746;
import org.apache.commons.lang3.mutable.MutableObject;
import org.jetbrains.annotations.NotNull;
import org.spongepowered.asm.mixin.Final;
import org.spongepowered.asm.mixin.Mixin;
import org.spongepowered.asm.mixin.Shadow;
import org.spongepowered.asm.mixin.Unique;
import org.spongepowered.asm.mixin.injection.At;
import org.spongepowered.asm.mixin.injection.Inject;
import org.spongepowered.asm.mixin.injection.Redirect;
import org.spongepowered.asm.mixin.injection.callback.CallbackInfo;
import org.spongepowered.asm.mixin.injection.callback.CallbackInfoReturnable;

import java.util.ArrayList;
import java.util.List;

import static eva.replacer.RePlacerClient.*;
import static eva.replacer.config.RePlacerConfig.*;
import static eva.replacer.util.BuildHolder.setBaseDir;
import static eva.replacer.util.BuildHolder.setFacing;
import static eva.replacer.util.BuildInProgress.*;

@Environment(EnvType.CLIENT)
@Mixin(class_636.class)
public abstract class MultiPlayerGameModeMixin {
    @Final @Shadow @NotNull private class_310 minecraft;
    @Unique @NotNull private final class_636 gameMode = (class_636) (Object) this;
    @Unique private class_746 player;
    @Unique private int waiter;

    @Inject(
            method = "tick",
            at = @At("HEAD")
    )
    private void placeAgain(CallbackInfo ci) {
        assert minecraft.field_1724 != null;
        if (player == null) player = minecraft.field_1724;
        if (rePlacing()) {
            if ((!isServer() || isReadyForBlock()) && player.method_5998(context().method_20287()).method_7909() instanceof class_1747 item) {
                if (!BuildInProgress.isEmpty()) {
                    waiter = 0;
                    RelPos pos = next();
                    if (player.method_56093(pos.pos(), 1)) {
                        assert context().method_8036() != null;
                        class_3965 result = BuildInProgress.place(pos, context().method_8036(), item);
                        if (!((BlockHitResultAccess) result).isMiss()) {
                            setReadyForBlock(false);
                            player.method_7259(class_3468.field_15372.method_14956(player.method_5998(context().method_20287()).method_7909()));
                        }
                    } /*else {
                        BuildInProgress.clear();
                        rePlacing(false);
                    }*/
                    return;
                } else if (!isClear()) BuildInProgress.clear();
                rePlacing(false);
            } else {
                if (waiter >= 5) {
                    if (!isClear()) BuildInProgress.clear();
                    rePlacing(false);
                    return;
                }
                waiter++;
            }
        }
    }

    @Redirect(
            method = "useItemOn",
            at = @At(
                    value = "INVOKE",
                    target = "Lnet/minecraft/client/multiplayer/MultiPlayerGameMode;startPrediction(Lnet/minecraft/client/multiplayer/ClientLevel;Lnet/minecraft/client/multiplayer/prediction/PredictiveAction;)V"
            )
    )
    private void reUseItemOn(
            class_636 instance,
            class_638 level,
            class_7204 action,
            @Local MutableObject<class_1269> mutableObject,
            @Local(argsOnly = true) class_1268 hand,
            @Local(argsOnly = true) class_3965 result
    ) {
        mutableObject.setValue(class_1269.field_5811);
        if (rePlacing()) return;

        if (!(player.method_5998(hand).method_7909() instanceof class_1747) || isReCording() || rePlacing() || !isHeldOrToggled()) {
            ((MultiPlayerGameModeInvoker) gameMode).invokeStartPrediction(this.minecraft.field_1687, i -> {
                mutableObject.setValue(((MultiPlayerGameModeInvoker) gameMode).invokePerformUseItemOn(player, hand, result));
                return new class_2885(hand, result, i);
            });
            if (mutableObject.getValue() instanceof class_1269.class_9860 && isReCording()) {
                class_1750 context = new class_1750(
                        player,
                        hand,
                        player.method_5998(hand),
                        result
                );
                if (buildSaver(context)) {
                    assert context.method_8036() != null;
                    context.method_8036().method_7353(class_2561.method_43470("Central block saved!"), true);
                }
            }
        } else {
            class_1750 context = new class_1750(
                    player,
                    hand,
                    player.method_5998(hand),
                    result
            );
            BuildHolder holder = handleGetBuild();
            if (isQuickAccess()) setQuickAccess(-1);
            if (holder == null) {
                RePlacerClient.LOGGER.info("Failed to get build!");
                player.method_7353(class_2561.method_43470("Failed to get build!"), true);
                player.method_7353(class_2561.method_43470("Writing default build."), false);
                return;
            }
            rePlacing(true);
            RelPos.setBase(context.method_17698(), context.method_8037());
            try {
                if (isSnap()) setBaseDir(context.method_8038());
                if (isSpin()) setFacing(player.method_5720(), context.method_8038());
                //Formerly compat mode
                BuildInProgress.pass(context);
                List<RelPos> poss = new ArrayList<>();
                holder.rotateEach(pos -> {
                    if (player.method_56093(pos.pos(), 1)
                            && level.method_8320(pos.pos()).method_26166(context)
                    )
                        poss.add(pos);
                });
                BuildHolder build = new BuildHolder(holder.firstDir(), holder.faceDir(), poss.toArray(new RelPos[0]));
                if (poss.isEmpty()) {
                    mutableObject.setValue(class_1269.field_5814);
                    return;
                }
                player.method_6104(hand);
                mutableObject.setValue(class_1269.field_5812);
                setReadyForBlock(true);
                build.handleSort(player).forEach(BuildInProgress::pass);
            } catch (Exception ignored) {
                mutableObject.setValue(class_1269.field_5814);
            }
        }
    }

    @Inject(
            method = "destroyBlock",
            at = @At(
                    value = "INVOKE",
                    target = "Lnet/minecraft/world/level/block/Block;destroy(Lnet/minecraft/world/level/LevelAccessor;Lnet/minecraft/core/BlockPos;Lnet/minecraft/world/level/block/state/BlockState;)V"
            )
    )
    private void removeBlock(class_2338 pos, CallbackInfoReturnable<Boolean> cir) {
        if (isReCording()) buildRemover(pos);
    }
}
